from tortue import *
import matplotlib.pyplot as plt

if getGraphics() :
	screenTurtle = turtle.Turtle()

area = 300
bornLimit = 20
startTurtlesCount = 20

def createRandomTurtle() :
	angle = random() * 2 * pi
	length = random() * area
	position = (length * cos(angle), length * sin(angle))
	angle = randint(-180, 180)
	color = (random(), random(), random())
	size = random() + 0.5
	speed = random() + 0.5
	pivot = random() + 0.5
	social = random() - 0.5
	sameSocial = random() - 0.5
	differentSocial = random() - 0.5
	survie = random() - 0.5
	age = 0
	maturity = randint(5, 15)
	return Tortue(
		position = position, 
		angle = angle, 
		color = color, 
		size = size, 
		speed = speed, 
		pivot = pivot,
		social = social,
		sameSocial = sameSocial,
		differentSocial = differentSocial,
		survie = survie,
		maturity = maturity,
		age = age)

if __name__ == "__main__":
	if getGraphics() :
		screenTurtle.getscreen().title("Simulation de tortues avec python 3 (création de la simulation)")
		
		screenTurtle.getscreen().bgcolor("#22dd55")
		screenTurtle.speed(100) # change la vitesse de la tortue
		screenTurtle.penup() # relève le stylo
		
		for c, a in [("#ddaa55", 1.12 * area), ("#5599dd", 0.84 * area), ("#ddaa55", 0.04 * area)] :
			screenTurtle.forward(a) # avance la tortue
			screenTurtle.left(90) # tourne la tortue pour préparer le cercle
			screenTurtle.pendown() # pose le style
			screenTurtle.color(c) # colorie la tortue
			screenTurtle.begin_fill() # se prépare à remplir le zone
			screenTurtle.circle(a) # tracer le cercle
			screenTurtle.end_fill() # fait le remplissage de la zone
			screenTurtle.penup() # relève le stylo
			screenTurtle.home() # va à la maison
		
		screenTurtle.forward(10000)
	
	asData = lambda t : "(vitesse : %g, taille : %g, social : (tous : %g, semblables : %g, différents : %g), survie : %g, maturity : %g)" % (t.speed, t.size, t.social, t.sameSocial, t.differentSocial, t.survie, t.maturity)
	
	turtles = list()
	for i in range(startTurtlesCount):
		t = createRandomTurtle()
		turtles.append(t)
		#print("+ Une tortue est née : %r" % asData(t))
	
	age = 0
	naissances = len(turtles)
	deces = 0
	evolution = list()
	evolutionNaissances = list()
	evolutionDeces = list()
	
	averageSpeeds = list()
	averageSizes = list()
	averageRobustness = list()
	averageBlue = list()
	averageGreen = list()
	averageRed = list()
	
	try :
		while len(turtles) > 0 :
			if getGraphics() :
				screenTurtle.getscreen().title("Simulation de tortues avec python 3 (age : %d, %d tortues)" % (age, len(turtles)))
			print("Age du monde : %d, %d tortues" % (age, len(turtles)))
			
			evolution.append(len(turtles))
			evolutionNaissances.append(naissances)
			evolutionDeces.append(deces)
			
			averageSpeeds.append(sum([e.speed for e in turtles]) / len(turtles))
			averageSizes.append(sum([e.size for e in turtles]) / len(turtles))
			averageRobustness.append(sum([e.robustness for e in turtles]) / len(turtles))
			averageRed.append(sum([e.color[0] for e in turtles]) / len(turtles))
			averageGreen.append(sum([e.color[1] for e in turtles]) / len(turtles))
			averageBlue.append(sum([e.color[2] for e in turtles]) / len(turtles))
			
			age += 1
			#bornLimit = age ** 1.25
			for t in turtles :
				t.action(turtles)
			ended = [t for t in turtles if t.health() <= 0.1]
			turtles = [t for t in turtles if t not in ended]
			deces += len(ended)
			for t in ended :
				#print("- Une tortue s'éteinte après %d années : %r" % (t.age, asData(t)))
				if getGraphics() :
					t.draw.speed(1000)
					t.draw.penup()
					t.draw.forward(10000)
			r = [(turtles[i], turtles[j]) for i in range(len(turtles)) for j in range(i + 1, len(turtles))]
			d = [(a, b) for a, b in r if not a.rest and not b.rest and a.age >= a.maturity and b.age >= b.maturity and squaredDistance(a.position, b.position) < 20 ** 2 and random() < 0.6]
			for first, second in d :
				if first.rest or second.rest : continue
				if(len(turtles) >= bornLimit) : continue
				babies = first + second
				naissances += len(babies)
				#for b in babies :
				#	print("+ Une tortue est née : %r" % asData(t))
				turtles.extend(babies)
		print("Le monde des tortues est arrivé à sa fin")
	except KeyboardInterrupt :
		print("Vous mettez fin au monde des tortues")
	evolution.append(len(turtles))
	evolutionNaissances.append(naissances)
	evolutionDeces.append(deces)
	plt.figure(1)
	plt.plot(evolution, "b", label="effectif")
	plt.plot(evolutionNaissances, "g", label="naissances")
	plt.plot(evolutionDeces, "r", label="décès")
	plt.legend()
	
	plt.figure(2)
	#plt.plot(averageRed, "red", label="rouge")
	#plt.plot(averageGreen, "green", label="vert")
	#plt.plot(averageBlue, "blue", label="bleu")
	plt.plot(averageSpeeds, "cyan", label="vitesse")
	plt.plot(averageRobustness, "purple", label="robustesse")
	plt.plot(averageSizes, "orange", label="taille")
	plt.legend()
	plt.show()