import turtle
from random import *
from math import *

doGraphicsDrawing = True

def getGraphics():
	return doGraphicsDrawing

colorIntoHexa = lambda c : "#" + "".join(["%02x" % (int(v * 255)) for v in c])
translate = lambda p, t : tuple(map(sum, zip(p, t)))
scale = lambda p, s : tuple([s * e for e in p])
radian = lambda d : ((d * pi) / 180)
squaredNorm = lambda p : sum([e * e for e in p])
norm = lambda p : sqrt(squaredNorm(p))
dot = lambda p, q : sum([a * b for a, b in zip(p, q)])
normalized = lambda p : scale(p, 1 / norm(p))
fromPolar = lambda a, r : (r * cos(a), r * sin(a))
squaredDistance = lambda p, q : squaredNorm([a - b for a, b in zip(p, q)])
distance = lambda p, q : sqrt(squaredDistance(p, q))
det = lambda p, q : p[0] * q[1] - p[1] * q[0]
diff = lambda p, q : tuple([a - b for a, b in zip(p, q)])

class Tortue :
	def __init__(self, **keys) :
		if getGraphics() :
			self.draw = turtle.Turtle()
			self.draw.speed(10000)
			self.draw.penup()
			self.draw.shape("turtle")
		
		self.speed = keys["speed"] if "speed" in keys else 1.
		self.size = keys["size"] if "size" in keys else 1.
		self.angle = keys["angle"] if "angle" in keys else 0
		self.position = keys["position"] if "position" in keys else (0, 0)
		self.color = keys["color"] if "color" in keys else (0.5, 0.5, 0.5)
		self.pivot = keys["pivot"] if "pivot" in keys else 1.
		self.robustness = (1.1 ** (-((1. - self.size) ** 2) / 1.)) * (1.1 ** (-((1. - self.speed) ** 2) / 0.8))
		self.social = keys["social"] if "social" in keys else 0.
		self.sameSocial = keys["sameSocial"] if "sameSocial" in keys else 0.
		self.differentSocial = keys["differentSocial"] if "differentSocial" in keys else 0.
		self.survie = keys["survie"] if "survie" in keys else 0.
		self.reproduction = keys["reproduction"] if "reproduction" in keys else 10.
		self.rest = False
		self.age = keys["age"] if "age" in keys else 0
		self.maturity = keys["maturity"] if "maturity" in keys else 3
		
		self.power = scale(self.color, 1 / sum(self.color))
		
		self.speed *= self.power[0] + 0.5
		self.robustness = self.robustness ** (1.5 - self.power[1])
		self.size *= self.power[2] + 0.5
		
		if getGraphics() :
			self.draw.left(self.angle)
			self.draw.goto(*self.position)
			self.draw.color(colorIntoHexa(self.color))
			self.draw.shapesize(self.size)
			self.draw.pensize(self.size)
			self.draw.speed(self.speed * 100)
			
			self.draw.pendown()
	
	def __add__(self, other) :
		babies = list()
		trials = min(int(min(self.reproduction, other.reproduction)), 10)
		for b in range(trials):
			speed = random() * self.speed + random() * other.speed
			size = random() * self.size + random() * other.size
			angle = randint(-180, 180)
			position = scale(translate(self.position, other.position), 0.5)
			color = tuple([min(random() * a + random() * b, 1) for a, b in zip(self.color, other.color)])
			pivot = random() * self.pivot + random() * other.pivot
			social = random() * self.social + random() * other.social
			sameSocial = random() * self.sameSocial + random() * other.sameSocial
			differentSocial = random() * self.differentSocial + random() * other.differentSocial
			survie = random() * self.survie + random() * other.survie
			reproduction = 1.5 * (random() * self.reproduction + random() * other.reproduction)
			maturity = random() * self.maturity + random() * other.maturity
			
			if speed < 0.1 or size < 0.1 or pivot < 0.1 : continue
			
			babies.append(Tortue(
				speed = speed,
				size = size,
				angle = angle,
				position = position,
				color = color,
				pivot = pivot,
				social = social,
				sameSocial = sameSocial,
				differentSocial = differentSocial,
				survie = survie,
				maturity = maturity
			))
		self.rest = True
		other.rest = True
		return babies
	
	def canReproducte(self):
		return self.reproduction > 0.
	
	def move(self, steps, turn) :
		self.position = self.moving(steps, turn)
		self.angle = (self.angle + turn) % 360
		if getGraphics() :
			self.draw.left(turn)
			self.draw.goto(*self.position)
	
	def environnement(self, others):
		maxDistance = 300
		currentDistance = norm(self.position)
		if(currentDistance > maxDistance):
			self.robustness *= 1.1 ** (-(maxDistance - currentDistance) ** 2 / 10000)
		self.reproduction *= self.robustness
		self.color = scale(self.color, self.robustness)
		self.color = tuple(map(lambda x : min(x, 1), self.color))
		if getGraphics() :
			self.draw.color(colorIntoHexa(self.color))
		self.age += 1
		vital = 10 * 1.1 ** (self.social + self.sameSocial + self.differentSocial)
		self.robustness *= 0.9999 ** (len([(vital ** 2 - squaredDistance(e.position, self.position)) / (vital ** 2) for e in others if squaredDistance(e.position, self.position) < vital ** 2]) ** 2)
		if self.rest :
			self.robustness *= self.robustness
			self.rest = False
	
	def health(self):
		return sum(self.color)
	
	def moving(self, steps, turn) :
		return translate(
			self.position, 
			fromPolar(radian(self.angle + turn), steps)
		)
	
	def randomMoving(self):
		steps = randint(0, 40) * self.speed * self.health()
		turn = randint(-30, 30) * self.pivot * self.health()
		return (steps, turn)
		
	def targetsMoving(self, targets):
		nb_trials = 10
		trials = [self.randomMoving() for t in range(nb_trials)]
		return min(trials, key = lambda x : sum([distance(self.moving(*x), t) * a for t, a in targets]))
	
	def action(self, others) :
		#steps, turn = self.randomMoving()
		steps, turn = self.targetsMoving(
			[((0, 0), self.survie * len(others))] 
			+ [(o.position, self.social) for o in others]
			+ [(o.position, self.sameSocial * distance(self.power, o.power)) for o in others]
			+ [(o.position, self.differentSocial * distance(diff(self.power, (1., 1., 1.)), o.power)) for o in others])
		
		turn %= 360
		if turn < -180 : turn += 360
		if turn > 180 : turn -= 360
		turnBounds = scale((-30, 30), self.pivot * self.health())
		turn = max(min(turn, max(turnBounds)), min(turnBounds))
		steps = min(max(steps, 0), 40 * self.speed * self.health())
		
		self.move(steps, turn)
		self.environnement(others)